import requests, re, json, os, sys, time
import xbmc, xbmcaddon, xbmcgui, xbmcplugin
from resources.lib.plugin import Plugin
from unidecode import unidecode
from datetime import datetime, timedelta
import calendar, inputstreamhelper
from jetextractors import extractors, extractor
from dateutil import tz
from resources.lib.plugin import run_hook

CACHE_TIME = 0  # change to wanted cache time in seconds

addon_fanart = xbmcaddon.Addon().getAddonInfo('fanart')
addon_icon = xbmcaddon.Addon().getAddonInfo('icon')
user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/63.0.3239.84 Safari/537.36'

class jetextractor(Plugin):
    name = "jetextractor"
    priority = 100

    def process_item(self, item):
        if "jetextractor" in item:
            extractor = item.get("jetextractor", "")
            item["link"] = "jetextractors/games/" + extractor
            item["is_dir"] = True
            thumbnail = item.get("thumbnail", "")
            fanart = item.get("fanart", "")
            list_item = xbmcgui.ListItem(item.get("title", item.get("name", "")), offscreen=True)
            list_item.setArt({"thumb": thumbnail, "fanart": fanart})
            item["list_item"] = list_item
            return item
        
    def play_video(self, video: str):
        item = json.loads(video)
        link = item.get("link")
        
        if link and link.startswith("jetextractor://"):
            links = link.replace("jetextractor://", "").split(",,")
            idx = 0
            if len(links) > 1:
                idx = xbmcgui.Dialog().select("Choose a link", links)
            if idx != -1:
                link = extractor.extract_m3u8(links[idx])
                if link.startswith("inputstream://"):
                    mpd_split = link.split("===")
                    mpd_url = mpd_split[0].replace("inputstream://", "")
                    license_key = mpd_split[1]
                    is_helper = inputstreamhelper.Helper('mpd', drm='widevine')
                    if not is_helper.check_inputstream():
                        sys.exit()
                    liz = xbmcgui.ListItem(item.get("title", mpd_url), path=mpd_url)
                    if int(xbmc.getInfoLabel('System.BuildVersion').split('.')[0]) >= 19: liz.setProperty('inputstream', 'inputstream.adaptive')
                    else: liz.setProperty('inputstreamaddon', 'inputstream.adaptive')
                    liz.setProperty('inputstream.adaptive.manifest_type', 'mpd')

                    if license_key != '':
                        liz.setProperty('inputstream.adaptive.license_type', 'com.widevine.alpha')
                        liz.setProperty('inputstream.adaptive.license_key', license_key)
                    liz.setMimeType('application/dash+xml')
                    liz.setContentLookup(False)
                    xbmcplugin.setResolvedUrl(int(sys.argv[1]), True, liz)
                    xbmc.Player().play(mpd_url, listitem=liz)
                    if len(mpd_split) > 2:
                        player = xbmc.Player()
                        for i in range(10):
                            if player.isPlayingVideo():
                                time.sleep(2)
                                player.seekTime(time.time() - float(mpd_split[2]) - 60)
                                break
                            else:
                                time.sleep(1)
                elif link.startswith("ffmpegdirect://"):
                    m3u8_url = link.replace("ffmpegdirect://", "")
                    liz = xbmcgui.ListItem(item.get("title", m3u8_url), path=m3u8_url)
                    if int(xbmc.getInfoLabel('System.BuildVersion').split('.')[0]) >= 19: liz.setProperty('inputstream', 'inputstream.ffmpegdirect')
                    else: liz.setProperty('inputstreamaddon', 'inputstream.ffmpegdirect')
                    liz.setProperty('inputstream.ffmpegdirect.is_realtime_stream', 'true')
                    liz.setProperty('inputstream.ffmpegdirect.stream_mode', 'timeshift')
                    liz.setProperty('inputstream.ffmpegdirect.manifest_type', 'hls')
                    liz.setMimeType('application/x-mpegURL')
                    xbmcplugin.setResolvedUrl(int(sys.argv[1]), True, liz)
                    xbmc.Player().play(m3u8_url, listitem=liz)
                else:
                    xbmc.Player().play(link)
                    
            return True
            

    def routes(self, plugin):
        @plugin.route("/jetextractors/games/<path:site>")
        def get_games(site):
            for module in extractors.__all__:
                if hasattr(module, "site_name") and hasattr(module, "get_games") and site == module.site_name:
                    games = module.get_games()
                    jen_list = []
                    for game in games:
                        jen_data = {
                            "title": "[COLORblue]%s[COLORwhite] |[B][I] %s[/B][/I]\n [COLORred]%s[/COLOR]" % (game["league"], game["title"], format_time(game["time"])),
                            "thumbnail": game["icon"],
                            "fanart": game["icon"],
                            "summary": game["title"],
                            "type": "item"
                        }
                        if len(game["links"]) > 0: jen_data["link"] = "jetextractor://" + ",,".join(game["links"])
                        jen_list.append(jen_data)
                    jen_list = [run_hook("process_item", item) for item in jen_list]
                    jen_list = [run_hook("get_metadata", item, return_item_on_failure=True) for item in jen_list]
                    run_hook("display_list", jen_list)


def format_time(date):
    return utc_to_local(date).strftime("%m/%d %I:%M %p") if date != "" else ""

def format_links(links):
    res = []
    for link in links:
        res.append("<sublink>jetextractor://%s</sublink>" % link)
    return "".join(res)

# https://stackoverflow.com/questions/4563272/convert-a-python-utc-datetime-to-a-local-datetime-using-only-python-standard-lib
def utc_to_local(utc_dt):
    timestamp = calendar.timegm(utc_dt.timetuple())
    local_dt = datetime.fromtimestamp(timestamp)
    assert utc_dt.resolution >= timedelta(microseconds=1)
    return local_dt.replace(microsecond=utc_dt.microsecond)


